// -*- C++ -*-

// Copyright 2006 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz 
// or its licensors, as applicable.
// Copyright 1995-2005 Thomas M. Breuel.
// 
// You may not use this file except under the terms of the accompanying license.
// 
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Project: iupr common header files
// File: coords.h
// Purpose: points and rectangles with integer coordinates
// Responsible: tmb
// Reviewer: 
// Primary Repository: 
// Web Sites: www.iupr.org, www.dfki.de

#ifndef h_coords_
#define h_coords_

#include "misc.h"

namespace iupr {

    struct point {
	int x,y;
	point() {}
	point(int x,int y)
	    :x(x),y(y) {
	}
    };

    struct rectangle {
	int x0,y0,x1,y1;
	rectangle()
            :x0(0),y0(0),x1(-1),y1(-1) {
	}
	rectangle(int x0,int y0,int x1,int y1)
	    :x0(x0),y0(y0),x1(x1),y1(y1) {
	}
	bool empty() {
	    return x0>=x1 || y0>=y1;
	}
	int width() {
	    return x1-x0;
	}
	int height() {
	    return y1-y0;
	}
	void include(int x,int y) {
	    if(empty()) {
		x0 = x;
		y0 = y;
		x1 = x+1;
		y1 = y+1;
	    } else {
		x0 = min(x,x0);
		y0 = min(y,y0);
		x1 = max(x+1,x1);
		y1 = max(y+1,y1);
	    }
	}
	bool contains(int x,int y) {
	    return x>=x0 && x<x1 && y>=y0 && y<y1;
	}
	void intersect(rectangle other) {
	    if(empty()) return;
	    x0 = max(x0,other.x0);
	    y0 = max(y0,other.y0);
	    x1 = min(x1,other.x1);
	    y1 = min(y1,other.y1);
	}
	void include(rectangle other) {
	    if(empty()) {
		*this = other;
	    } else {
		x0 = min(x0,other.x0);
		y0 = min(y0,other.y0);
		x1 = max(x1,other.x1);
		y1 = max(y1,other.y1);
	    }
	}
	rectangle intersection(rectangle other) {
	    if(empty()) return *this;
	    return rectangle(
		max(x0,other.x0),
		max(y0,other.y0),
		min(x1,other.x1),
		min(y1,other.y1));
	}
	rectangle inclusion(rectangle other) {
	    if(empty()) return other;
	    return rectangle(
		min(x0,other.x0),
		min(y0,other.y0),
		max(x1,other.x1),
		max(y1,other.y1));
	}
    };

}

#endif
