// Copyright 2006 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz 
// or its licensors, as applicable.
// 
// You may not use this file except under the terms of the accompanying license.
// 
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Project: iupr common header files
// File: idmap.h
// Purpose: implementation of idmap as a set of points
// Responsible: mezhirov
// Reviewer: 
// Primary Repository: 
// Web Sites: www.iupr.org, www.dfki.de



#ifndef h_idmap_
#define h_idmap_

#include "hash.h"

namespace iupr {

class idmap {
    inthash< autoref< inthash<int> > > ids_to_segments;
    inthash< autoref< inthash<int> > > segments_to_ids;
    int max_segment;

public:
    idmap() : max_segment(-1) {}
    void set_max_segment(int new_max_segment) {
        max_segment = new_max_segment;
    }
    void segments_of_id(intarray &result, int id) {
        ids_to_segments(id)->keys(result);
    }
    void ids_of_segment(intarray &result, int segment) {
        segments_to_ids(segment)->keys(result);
    }
    void ids(intarray &result) {
        ids_to_segments.keys(result);
    }
    int ids_count() {
        intarray temp;
        ids(temp);
        return temp.length();
    }
    int segments_count() {
        intarray temp;
        segments(temp);
        return temp.length();
    }
    void segments(intarray &result) {
        segments_to_ids.keys(result);
    }
    void bind(int id, int segment) {
        ASSERT(max_segment == -1 || segment <= max_segment);
        (*ids_to_segments(id))(segment) = 1;
        (*segments_to_ids(segment))(id) = 1;
    }
    void clear() {
        ids_to_segments.dealloc();
        segments_to_ids.dealloc();
    }
};

}

#endif
