// -*- C++ -*-

// Copyright 2006 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz 
// or its licensors, as applicable.
// Copyright 1995-2005 Thomas M. Breuel
// 
// You may not use this file except under the terms of the accompanying license.
// 
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Project: imgio -- reading and writing images
// File: autoinvert.cc
// Purpose: automatically invert document images to be white=0
// Responsible: tmb
// Reviewer: 
// Primary Repository: 
// Web Sites: www.iupr.org, www.dfki.de

#include "narray.h"
#include "env.h"

namespace iupr {

    param_int maybe_invert("maybe_invert",2,
			   "setting for autoinvert; 0: don't invert, 1: do invert, "
			   "2: invert to make majority of pixels black");

    void autoinvert(bytearray &image) {
	if(maybe_invert==0) return;
	bool invert = false;
	if(maybe_invert==1) {
	    invert = 1;
	} else if(maybe_invert==2) {
	    int nonzero = 0;
	    for(int i=0;i<image.dim(0);i++)
		for(int j=0;j<image.dim(1);j++)
		    if(image(i,j)) nonzero++;
	    invert = (nonzero*1.0/image.dim(0)/image.dim(1)>0.5);
	} else {
	    throw "bad value for maybe_invert";
	}
	if(invert) {
	    for(int i=0;i<image.length1d();i++)
		image.at1d(i) = 0xff * !image.at1d(i);
	} else {
	    for(int i=0;i<image.length1d();i++)
		image.at1d(i) = 0xff * !!image.at1d(i);
	}
    }

    void autoinvert(intarray &image) {
	if(maybe_invert==0) return;
	bool invert = false;
	if(maybe_invert==1) {
	    invert = 1;
	} else if(maybe_invert==2) {
	    int nonzero = 0;
	    for(int i=0;i<image.dim(0);i++)
		for(int j=0;j<image.dim(1);j++)
		    if(image(i,j)) nonzero++;
	    invert = (nonzero*1.0/image.dim(0)/image.dim(1)>0.5);
	} else {
	    throw "bad value for maybe_invert";
	}
	if(invert) {
	    for(int i=0;i<image.length1d();i++)
		image.at1d(i) = 0xffffff * !image.at1d(i);
	} else {
	    for(int i=0;i<image.length1d();i++)
		image.at1d(i) = 0xffffff * !!image.at1d(i);
	}
    }

}
