// -*- C++ -*-

// Copyright 2006 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz 
// or its licensors, as applicable.
// Copyright 1995-2005 Thomas M. Breuel
// 
// You may not use this file except under the terms of the accompanying license.
// 
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Project: imglib -- image processing library
// File: imgops.h
// Purpose: interface to corresponding .cc file
// Responsible: tmb
// Reviewer: 
// Primary Repository: 
// Web Sites: www.iupr.org, www.dfki.de
#ifndef h_imgops__
#define h_imgops__

namespace iupr {

template <class T> void reverse(narray<T> &);
template <class T,class S> void getd0(narray<T> &image,narray<S> &slice,int index);
template <class T,class S> void getd1(narray<T> &image,narray<S> &slice,int index);
template <class T,class S> void putd0(narray<T> &image,narray<S> &slice,int index);
template <class T,class S> void putd1(narray<T> &image,narray<S> &slice,int index);
float gradx(floatarray &image,int x,int y);
float grady(floatarray &image,int x,int y);
float gradmag(floatarray &image,int x,int y);
float gradang(floatarray &image,int x,int y);

template <class T>
inline T &xref(narray<T> &a,int x,int y) {
    if(x<0) x = 0; else if(x>=a.dim(0)) x = a.dim(0)-1;
    if(y<0) y = 0; else if(y>=a.dim(1)) y = a.dim(1)-1;
    return a.unsafe_at(x,y);
}

template <class T,class V> void addscaled(narray<T> &,narray<T> &,V,int,int);
template <class T> void tighten(narray<T> &image);
template <class T> void circ_by(narray<T> &image,int dx,int dy,T value=0);
template <class T> void shift_by(narray<T> &image,int dx,int dy,T value=0);
template <class T> void pad_by(narray<T> &image,int px,int py,T value=0);
template <class T,class S> void extract_subimage(narray<T> &subimage,narray<S> &image,int x0,int y0,int x1,int y1);
template <class T> void resize_to(narray<T> &image,int w,int h,T value=0);
void compose_at(bytearray &image,bytearray &source,int x,int y,
		int value,int conflict);

template <class T,class U,class V,class W>
    void ifelse(narray<T> &dest,narray<U> &cond,narray<V> &iftrue,narray<W> &iffalse);
void blend(floatarray &dest,floatarray &cond,floatarray &iftrue,floatarray &iffalse);

}

#endif
