// Copyright 2006 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz 
// or its licensors, as applicable.
// 
// You may not use this file except under the terms of the accompanying license.
// 
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Project: ocr-adaptive
// File: ocr-adapters.h
// Purpose: interface to ocr-adapters.cc
// Responsible: mezhirov
// Reviewer: 
// Primary Repository: 
// Web Sites: www.iupr.org, www.dfki.de


#ifndef h_ocr_adapters_
#define h_ocr_adapters_

namespace iupr {
    /// Create an adapter of ISimpleLineOCR to ISegmentedLineOCR.
    /// It runs ISimpleLineOCR on the input without segmentation,
    /// then heuristically matches bounding boxes returned by ISimpleLineOCR
    /// with the segments (in the hope that there is 1-to-many correspondence).
    ISegmentedLineOCR *make_adapter(ISimpleLineOCR *);

    /// Create an adapter of ISimpleRetrainableOCR to ISegmentedRetrainableOCR.
    /// When recognizing, it does the same as the non-retrainable adapter.
    /// When training, it chooses only those characters for training
    /// that have cost below training_threshold.
    ISegmentedRetrainableOCR *make_adapter(ISimpleRetrainableOCR *, float training_threshold);
};

#endif
