// Copyright 2006 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz 
// or its licensors, as applicable.
// 
// You may not use this file except under the terms of the accompanying license.
// 
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Project:
// File: 
// Purpose: 
// Responsible: 
// Reviewer: 
// Primary Repository: 
// Web Sites: 

#include "iupr.h"
#include "ocrinterfaces.h"
#include "langmod-aspell.h"
#include "langmod-ispell.h"
#include "langmod-shortest-path.h"

using namespace iupr;

void langmod_put(ILanguageModel &langmod, const char *s) {
    int n = strlen(s);
    langmod.start_chunk(n + 1);
    for(int i = 0; i < n; i++)
        langmod.add_transition(i, i + 1, s[i], 1, i);
}

bool is_dictionary_word(ILanguageModel &langmod, const char *s) {
    langmod_put(langmod, s);
    langmod.compute(1);
    if (!langmod.nresults())
        return false;

    intarray result;
    floatarray costs;
    intarray ids;
    intarray states;
    langmod.nbest(result, costs, ids, states, 0);
    return sum(costs) < 10 * costs.length();
}

void test_spell(ILanguageModel &langmod) {
    CHECK(is_dictionary_word(langmod, "open"));
    CHECK(is_dictionary_word(langmod, "source"));
    CHECK(is_dictionary_word(langmod, "freedom"));
    CHECK(!is_dictionary_word(langmod, "preved"));
    CHECK(!is_dictionary_word(langmod, "medved"));
    CHECK(!is_dictionary_word(langmod, "OCRopusRULEZ"));
}

int main() {
    autodel<ILanguageModel> langmod(make_IspellLanguageModel(make_ShortestPathLanguageModel()));
    test_spell(*langmod);
    langmod = make_AspellLanguageModel(make_ShortestPathLanguageModel());
    test_spell(*langmod);
}
