// -*- C++ -*-

// Copyright 2006 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz 
// or its licensors, as applicable.
// 
// You may not use this file except under the terms of the accompanying license.
// 
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Project: ocr-layout-rast - Extract textlines from a document image using RAST
// File: ocrcharstats.h
// Purpose: Header file declaring data structures for computing document
//          statistics
// Responsible: Faisal Shafait (faisal.shafait@dfki.de)
// Reviewer: 
// Primary Repository: 
// Web Sites: www.iupr.org, www.dfki.de

#ifndef h_ocrcharstats__
#define h_ocrcharstats__

#include "ocrinterfaces.h"
#include "rect.h"
#include "struct.h"
#include "heap.h"
#include "iarith.h"
#include "iupr.h"

namespace iupr {

    //////////////////////////////////////////////////////////////////////////
    ///
    /// \struct CharStats
    /// Purpose: Character bounding boxes and statistics extracted from them
    ///
    //////////////////////////////////////////////////////////////////////////
    
    struct CharStats {
        int    img_height;
        int    img_width;
        int    xheight;
        int    char_spacing;
        int    word_spacing;
        int    line_spacing;
        narray<Rect> char_boxes;
        narray<Rect> dot_boxes;

        CharStats();
        CharStats(CharStats &c);
        ~CharStats();
        void print();
        void get_char_boxes(narray<rectangle> &concomps);
        void calc_char_stats();
        void calc_char_stats(narray<Rect> &cboxes);
        void calc_char_stats_for_one_line();
        void calc_char_stats_for_one_line(narray<Rect> &cboxes);
        void sort_boxes_by_x0(narray<Rect> &boxes);
        void sort_boxes_by_y0(narray<Rect> &boxes);
        void calc_hist(floatarray &distances, floatarray &hist);
        void find_modes(floatarray &hist, int &mode1, int &mode2);
        int  estimate_xheight(narray<Rect> &bboxes);

    private:
        inline bool x_overlap (Rect a, Rect b);
        inline bool y_overlap (Rect a, Rect b);
    };
    CharStats *make_CharStats();
    CharStats *make_CharStats(CharStats &c);

}

#endif
