// -*- C++ -*-

// Copyright 2006 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz 
// or its licensors, as applicable.
// 
// You may not use this file except under the terms of the accompanying license.
// 
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Project:
// File: 
// Purpose: 
// Responsible: 
// Reviewer: 
// Primary Repository: 
// Web Sites: 


#include "iupr.h"
#include "imgio.h"
#include "ocr-utils.h"
#include "tesseract.h"

using namespace iupr;

void test_ocr_result(ucarray &ocr_result, const char *gt) {
    int gt_len = strlen(gt);
    for(int i = 0; i < ocr_result.length(); i++) {
        if(i >= gt_len || ocr_result[i] != gt[i]) {
            fprintf(stderr, "mistaken char #%d for %lc in %s", i, ocr_result[i], gt);
        }
    }
}

static void test_line_ocr(ISimpleLineOCR &ocr) {
    stdio filelist("test-list", "r");
    char in_path[1000];
    while(fscanf(filelist, "%999s", in_path) == 1) {
        char gt_path[1000];
        fscanf(filelist, "%999s", gt_path);
        char gt[1000];
        stdio gt_file(gt_path, "r");
        fgets(gt, sizeof(gt), gt_file);
        bytearray image;
        ucarray result;
        floatarray costs;
        narray<rectangle> bboxes;
        read_image_gray(image, in_path);
        ocr.recognize_gray(result, costs, bboxes, image);
        test_ocr_result(result, gt);
        binarize_by_threshold(image);
        ocr.recognize_binary(result, costs, bboxes, image);
        test_ocr_result(result, gt);
    }
}

int main(int argc, char **argv) {
    autodel<ISimpleRetrainableOCR> ocr(make_tesseract(argv[0]));
    ocr->description();
    ocr->init(NULL);
    intarray alice;
    read_png_rgb(alice, stdio("alice.png", "rb"));
    bytearray alice_bin;
    forget_segmentation(alice_bin, alice);
    ucarray alice_text(5);
    alice_text[0] = 'A';
    alice_text[1] = 'l';
    alice_text[2] = 'i';
    alice_text[3] = 'c';
    alice_text[4] = 'e';
    CHECK(ocr->supports_char_training() && !ocr->supports_line_training());
    try{ocr->train_binary(alice_text, alice_bin); CHECK(0);} catch (...) {}
    try{ocr->train_gray  (alice_text, alice_bin); CHECK(0);} catch (...) {}
    test_line_ocr(*ocr);
    ocr->start_training();
    ocr->train_binary_chars(alice_text, alice);
    ocr->train_gray_chars(alice_text, alice, alice_bin);
    ocr->start_recognizing();
    test_line_ocr(*ocr);
}
